import os
import sys
from math import fabs

import pygame
from pygame.locals import *

from globals import *
from helper import *
from screen import scr
import tiles
import screen
import map
import spritesheet
from sprite import enemies,  projectiles, itemz
import sprite
import menu
import font
import status
import player

# stuff that needs to happen in rough priority order with estimates

# - player abilities 
# LEGS: (exclusive)
#	- dash *
#	- high jump *
#	- dodge roll *
# LEFT ARM:
#	- wall grab (+jump off) *
# TORSO:
#	- heat resist
# CORE
#	- HP / HP regen boost
#	- ammo / ammo regen boost
# RIGHT ARM: (exclusive except rate boost)
#	- spread shot
#	- rapid shot
#	- beam
# 	- flamethrower
#	- missile launcher

# working by 5am Fri?

# - additional player sprites for anim. frames (06:00) finished 7am
# - implement player abilities (07:00) finished 10am
# - finish tileset 1 1hr finished 11:30am
# - debug map editor .5hr finished noon

# - add climbing
# - make 2 dungeon maps for set 1 3hr

# - generalize enemy class 
#	- add 4+ new enemies
#	- add a boss enemy 3hr

# - generalize shot class 
#	- add weapons as above 1.5hr



# ------ this stuff isn't likely to get done :(

# - finish menu, text display, etc.
# - player robot upgrade menu 1.5hr

# - "storyline" stuff 1hr
# - title screen / main menu? 1.5hr

# - load/save game .5hr

# - make 1 or 2 more tilesets (inferno, forest, ice, ...?) 3hr

# - moving platforms 1.5hr

# - add 2-3 dungeon maps per tileset 2hr

# - add teleport dodge ability somehow?
# - add attack dash?
# - new enemies for dungeons?

# - projectiles (all sprites?) hit walls based on hitbox
# - generalize item class

# - optimize scrolling, it currently chews way too much cpu
# 	- overhaul tile rendering to fix?
# - may want to change rendering to eliminate the tile buffer, scroll by blitting and redrawing edges?
#	- would still need a buffer, maybe have two that alternate and copy the current one onto the display

clock = pygame.time.Clock()

def main():
	pygame.init()
	pygame.display.set_mode((320,240), pygame.FULLSCREEN | pygame.HWSURFACE | pygame.DOUBLEBUF)
	#pygame.display.set_mode((320,240))
	pygame.mouse.set_visible(False)
	
	#do_mapedit()
	
	tiles.load_tilesets()
	map.load_maps()
	font.load_fonts()
	spritesheet.load_spritesheets()
	scr.map = map.maps[0]
	
	plr = player.Player()
	plr.location = (0, 0)
	scr.scroll_by((-120, -120))
	
	user_input = 0
	jump = 0.0
	
	done = False
	
	while not done:
		
		for event in pygame.event.get():
			if event.type == KEYDOWN:
				if event.key == Key.quit:
					done = True
				elif event.key == pygame.K_s:
					temp = pygame.display.get_surface().copy()
					pygame.image.save(temp, 'scr.TGA')
				elif event.key == pygame.K_m:
					menu.TownMenu(0).go()
			elif event.type == KEYUP:
				pass
		
		
		#if is_far_tp(scr.map.spawns[0].location, plr.location):
		#	print scr.map.spawns[0].location, (plr.location[0] / TILEX, plr.location[1] / TILEY)
		#else:
		#	print "near"
		for sp in scr.map.spawns:
			sp.check_far(plr)
			if sp.enemy != None:
				sprite.enemies.add(sp.enemy)
				sp.enemy = None
		
		for p in projectiles:
			if is_far_pp(plr.location, p.location):
				# too far off screen
				p.kill()
			if p.hostile:
				if p.collides_with(plr):
					p.hit_target(plr)
			else:
				for e in enemies:
					if p.collides_with(e):
						p.hit_target(e)
			p.act()
		
		for i in itemz:
			if i.collides_with(plr):
				i.collected()
			i.act()
		
		for e in enemies:
			if e.collides_with(plr) and e.hp > 0:
				plr.hit(e)
				
			if is_far_pp(e.location, plr.location):
				# too far off screen
				e.kill()
			elif e.wait == 0:
				e.act(plr)
			else:
				e.wait -= 1
			
			projectiles.add(e.children.sprites())
			itemz.add(e.drop.sprites())
		
		old_pos = plr.location
		plr.act(pygame.key.get_pressed())
		projectiles.add(plr.children.sprites())
		scr.scroll_by((plr.location[0] - old_pos[0], plr.location[1] - old_pos[1]))
		
		
		if plr.life <= 0:
			# ur dead
			done = True
		
		scr.draw_tiles()
		plr.draw(scr.get_surface())
		enemies.draw(scr.get_surface())
		projectiles.draw(scr.get_surface())
		itemz.draw(scr.get_surface())
		scr.draw_status_area(plr.life, status.maxlife, plr.ammo, status.maxammo)
		scr.done()
		
		clock.tick(frame_rate)
	
	print "Thanks for playing!"

def do_mapedit():
	tiles.load_tilesets()
	map.load_maps()
	font.load_fonts()
	
	spritesheet.load_spritesheets()
	if len(map.maps) > 0:
		scr.map = map.maps[0]
		mapnum = 0
	else:
		scr.map = map.new_map()
		mapnum = -1
	
	for sp in scr.map.spawns:
		sp.enemy = None
	
	done = False
	tilenum = 61
	set = 0
	
	while not done:
		scroll = (0, 0)
		ctile = 0
		for event in pygame.event.get():
			if event.type == KEYDOWN:
				if event.key == Key.up:
					scroll = (0, -16)
				elif event.key == Key.down:
					scroll = (0, 16)
				elif event.key == Key.left:
					scroll = (-16, 0)
				elif event.key == Key.right:
					scroll = (16, 0)
				elif event.key == Key.select:
					scr.set_at_cursor(tilenum)
				elif event.key == pygame.K_q:
					ctile = -1
				elif event.key == pygame.K_a:
					ctile = -5
				elif event.key == pygame.K_z:
					ctile = -20
				elif event.key == pygame.K_w:
					ctile = 1
				elif event.key == pygame.K_s:
					ctile = 5
				elif event.key == pygame.K_x:
					ctile = 20
				elif event.key == pygame.K_e:
					scr.set_enemy_spawn(sprite.Eyeball)
				elif event.key == pygame.K_d:
					scr.set_doorway()
				elif event.key == Key.quit:
					done = True
				elif event.key == pygame.K_p: # p is for pickle = save
					if mapnum == -1:
						mapnum = map.save_new_map()
						scr.map = map.maps[mapnum]
					else:
						map.save_maps()
				elif event.key == pygame.K_n:
					mapnum = -1
					scr.map = map.new_map()
					scr.need_refresh = True
				elif event.key == pygame.K_i:
					mapnum = (mapnum - 1) % len(map.maps)
					scr.map = map.maps[mapnum]
					scr.need_refresh = True
				elif event.key == pygame.K_o:
					mapnum = (mapnum + 1) % len(map.maps)
					scr.map = map.maps[mapnum]
					scr.need_refresh = True
				elif event.key == pygame.K_c:
					print scr.get_cursor_loc()
				elif event.key == pygame.K_v:
					for m in map.maps:
						print m.num
						for d in m.doors:
							#scr.screen_position(d.location)
							print d.location,  d.target.location
				elif event.key == pygame.K_b:
					for m in map.maps:
						m.doors = []
						
		
		if scroll != (0, 0):
			scr.scroll_by(scroll)
		
		tilenum = (tilenum + ctile) % tiles.tilesets[set].count()
		
		scr.draw_tiles()
		scr.draw_features()
		scr.draw_cursor()
		pygame.draw.rect(pygame.display.get_surface(), (255, 0, 255), pygame.Rect(4, 4, 18, 18))
		pygame.display.get_surface().blit(tiles.tilesets[set].surf, (5, 5), tiles.tilesets[set].tile(tilenum))
		
		pygame.display.flip()
		
		clock.tick(frame_rate)
	