import pygame
from pygame.locals import *

from globals import *
from helper import *
from screen import scr
import screen
import spritesheet
import status
import map

class XSprite(pygame.sprite.Sprite):
	def __init__(self, sheet, size):
		pygame.sprite.Sprite.__init__(self)
		self.sheet = spritesheet.spritesheets[sheet]
		self.size = size
		self.frame = 0
		self.location = (0, 0)
		self.wait = 0
		self.cycle = 5
		self.vx = 0
		self.vy = 0
		self.ax = 0
		self.ay = 0
		self.action = self.ac_nothing
		self.automove = True
		self.dieatwall = False
	
	def set_vel(self, x, y):
		self.vx = x
		self.vy = y
	
	def set_acc(self, x, y):
		self.ax = x
		self.ay = y
	
	def act(self, *params):
		self.action(*params)
		if self.wait != None:
			self.wait = self.cycle
		if self.automove:
			self.vx += self.ax
			self.vy += self.ay
			if not self.move_max((int(self.vx), int(self.vy))):
				if self.dieatwall:
					self.action = self.ac_die
				
			
			#loc = self.get_rectangle().move(self.vx, self.vy)
			#self.location = (loc.left, loc.top)
		
	def ac_nothing(self, *params):
		pass
	
	def ac_die(self, *params):
		self.kill()
	
	def draw(self, surface):
		surface.blit(self.sheet.surf, scr.screen_position(self.location), self.sheet.frame(self.size, self.frame))
	
	def get_rectangle(self):
		map_px = scr.map.size[0] * TILEX
		map_py = scr.map.size[1] * TILEY
		px = self.location[0] % map_px
		py = self.location[1] % map_py
		if (self.location[0] - (scr.mapx * TILEX + scr.offx)) %  map_px > (map_px / 2):
			px -= map_px
		py = self.location[1]
		if (self.location[1] - (scr.mapy * TILEY + scr.offy)) %  map_py > (map_py / 2):
			py -= map_py
		return pygame.Rect((px, py), (self.size[0] * TILEX, self.size[1] * TILEY))

	def move(self, delta):
		self.location = sum_tuple(self.location, delta)
	
	def obstructed(self, delta):
		return scr.map.obstructed_in(self.get_rectangle().move(delta))
	
	def can_climb(self):
		return scr.map.tile(((self.location[0] + 8) / TILEX, (self.location[1] + 12) / TILEY)) == 11
	
	def can_fall(self):
		return not self.obstructed((0, 1))
	
	def at_ledge(self, dx):
		check = self.get_rectangle().move(dx, 2)
		return (not self.can_fall() and not scr.map.obstructed_at((check.right, check.bottom)))
	
	def move_max(self, delta):
		xhit = True
		yhit = True
		if delta[0] != 0:
			xhit = self.move_x_max(delta[0])
		if delta[1] != 0:
			yhit = self.move_y_max(delta[1])
		return (xhit and yhit)
	
	def move_x_max(self, dx):
		if not self.obstructed((dx, 0)):
			self.move((dx, 0))
			return True
		else:
			if dx < 0:
				inc = -1
			else:
				inc = 1
			for count in range(dx * inc):
				if self.obstructed((inc, 0)):
					return False
				else:
					self.move((inc, 0))
	
	def move_y_max(self, dy):
		if not self.obstructed((0, dy)):
			self.move((0, dy))
			return True
		else:
			if dy < 0:
				inc = -1
			else:
				inc = 1
			for count in range(dy * inc):
				if self.obstructed((0, inc)):
					return False
				else:
					self.move((0, inc))
	
	def collides_with(self, other):
		r1 = self.get_rectangle()
		r2 = other.get_rectangle()
		return self.hitbox.move(r1.left, r1.top).colliderect(other.hitbox.move(r2.left, r2.top)) 

class XGroup(pygame.sprite.Group):
	def __init__(self, *spr):
		pygame.sprite.Group.__init__(self, *spr)
	
	def draw(self, surface):
		for spr in self.sprites():
			spr.draw(surface)

class Shot(XSprite):
	def __init__(self, delta, sheet, size):
		XSprite.__init__(self, sheet, size)
		self.action = self.ac_nothing
		self.fade = 320
		self.vx = delta[0]
		self.vy = delta[1]
		self.dieatwall = True
		if self.vx > 0:
			self.frame = 1
		else:
			self.frame = 0
	
	def act(self, *params):
		self.fade -= 1
		if self.fade == 0:
			self.action = self.ac_die
		XSprite.act(self, *params)
	
	def hit_target(self, target):
		target.hit(self)
		self.action = self.ac_die

class PlayerNormalShot(Shot):
	def __init__(self, delta):
		Shot.__init__(self, delta, 'S01', (1,1))
		self.hitbox = pygame.Rect(0, 6, 16, 4)
		self.damage = 2
		self.hostile = False
		if self.vx > 0:
			self.frame = 1
		else:
			self.frame = 0

class EyeShot(Shot):
	def __init__(self, delta):
		Shot.__init__(self, delta, 'S01', (2,1))
		self.hitbox = pygame.Rect(0, 6, 16, 4)
		self.damage = 4
		self.hostile = True
		self.frame = 1

class Item(XSprite):
	def __init__(self):
		XSprite.__init__(self, 'S01', (1,1))
		self.hitbox = pygame.Rect(0, 0, 16, 16)
		self.framecount = 0
		self.action = self.ac_stuff
		self.vy = 2
	
	def collected(self):
		status.parts += 1
		self.action = self.ac_die
	
	def ac_stuff(self):
		self.framecount += 1
		if self.framecount < 200:
			self.frame = 20 + (self.framecount / 4 ) % 2
		elif self.framecount < 300:
			self.frame = 20 + ((self.framecount / 4 ) % 2) * 2
		else:
			self.action = self.ac_die


class Enemy(XSprite):
	def __init__(self, sheet, size):
		XSprite.__init__(self, sheet, size)
		self.action = self.ac_start
		self.children = XGroup()
		self.drop = XGroup()
	
	def ac_die(self, plr):
		itam = Item()
		itam.location = self.location
		self.drop.add(itam)
		XSprite.ac_die(self)
	
	def hit(self, thing):
		self.hp -= thing.damage
		if self.hp <= 0:
			self.action = self.ac_die
		else:
			self.cycle = 2
			self.frame = 80
			self.action = self.ac_ouch
	
	def seek_to(self, plr):
		dx = (plr.location[0] - self.location[0]) % (scr.map.size[0] * TILEX)
		dy = (plr.location[1] - self.location[1]) % (scr.map.size[1] * TILEY)
		if dx > (scr.map.size[0] * TILEX * 0.5):
			dx -= (scr.map.size[0] * TILEX)
		if dy > (scr.map.size[1] * TILEX * 0.5):
			dy -= (scr.map.size[1] * TILEY)
		return (dx, dy)

class Eyeball(Enemy):
	def __init__(self):
		Enemy.__init__(self,'E01', (1,1))
		self.hitbox = pygame.Rect(0, 0, 16, 16)
		self.damage = 2
		self.hp = 16
		self.framecount = 0
	
	def ac_start(self, plr):
		self.dir = -1
		self.framedir = 0
		self.frame = 1
		self.cycle = 2
		self.action = self.ac_walk
	
	def ac_walk(self, plr):
		self.vx = self.dir * 3
		self.framecount = (self.framecount + 1) % 40
		
		if self.framecount < 5:
			self.frame = 21 + self.framedir + (self.framecount % 5)
		elif self.dir < 0:
			self.frame = 1
		else:
			self.frame = 4
		
		seek = self.seek_to(plr)
		
		if seek[1] > -24 and seek[1] < 8:
			if seek[0] * self.dir < 0:
				self.vx *= -1 
				self.vy = 0
				if self.dir == 1:
					self.framecount = 4
				else:
					self.framecount = 1
				self.action = self.ac_turn
				return
			else:
				self.framecount = 0
				self.vy = 0
				self.vx = 0
				self.action = self.ac_shoot
				return
		
		if self.obstructed((self.vx * 2, 0)) and not self.obstructed((self.vx, -16)):
			self.vy = -3
			return
		elif self.obstructed((self.vx * 2, 0)) and not self.obstructed((self.vx, 8)):
			self.vy = 3
			return
		elif self.obstructed((self.vx * 2, 0)):
			self.vx *= -1 
			self.vy = 0
			if self.dir == 1:
				self.framecount = 4
			else:
				self.framecount = 1
			self.action = self.ac_turn
			return
		elif self.obstructed((0, 8)):
			self.vy = -1
		elif not self.obstructed((0, 24)):
			self.vy = 1
		elif self.vy == 0:
			self.vy = -1
	
	def ac_turn(self, plr):
		self.frame = 0 + self.framecount
		self.framecount -= self.dir
		if self.framecount < 1 or self.framecount > 4:
			self.dir = -self.dir
			self.framedir = (self.dir + 1) * 10
			self.framecount = 0
			self.action = self.ac_walk
	
	def ac_shoot(self, plr):
		seek = self.seek_to(plr)
		
		if self.framecount > 0:
			self.framecount += 1
			if self.framecount == 5:
				shot = EyeShot((self.dir * 12, 0))
				shot.location = sum_tuple(self.location, ((48 * self.dir) - 32, 0))
				self.children.add(shot)
			
			if self.framecount < 8:
				self.frame = 59 + self.framedir + self.framecount
				return
			else:
				self.framecount = 0
		
		if self.dir < 0:
			self.frame = 1
		else:
			self.frame = 4
		
		if seek[1] < -40 or seek[1] > 24:
			self.action = self.ac_walk
			return
		elif seek[1] < -16:
			self.vy = -2
		elif seek[1] > 0:
			self.vy = 2
		elif seek[0] * self.dir < -1:
			self.vx = 0
			self.vy = 0
			if self.dir == 1:
				self.framecount = 4
			else:
				self.framecount = 1
			self.action = self.ac_turn
		elif seek[0] > -100 and seek[0] < 100:
			self.vx = self.dir * -4
		elif seek[0] < -250 and seek[0] > 250:
			self.vx = self.dir * 4
		else:
			self.vx = 0
		
		self.framecount = 1
	
	def ac_ouch(self, plr):
		self.frame = 26 + self.framedir
		self.action = self.ac_walk

class Slime(Enemy):
	def __init__(self):
		Enemy.__init__(self,'E01', (1,1))
		self.hitbox = pygame.Rect(0, 8, 16, 8)
		self.damage = 2
		self.hp = 8
	
	def ac_start(self, plr):
		self.cycle = 10
		self.set_vel(4, 0)
		self.set_acc(0, 0)
		self.action = self.ac_walk
	
	def ac_walk(self, plr):
		if self.can_fall():
			self.cycle = 0
			self.action = self.ac_fall
		self.frame = (self.frame + 20) % 80
		
		if self.obstructed((self.vx, 0)) or self.at_ledge(self.vx):
			self.vx = -self.vx
	
	def ac_fall(self, plr):
		if self.vy < TERM_VEL:
			self.set_vel(0, self.vy + GRAVITY)
		self.move_y_max(int(self.vy))
		if not self.can_fall():
			self.action = self.ac_start
	
	def ac_ouch(self, plr):
		self.cycle = 10
		self.frame = 0
		self.action = self.ac_walk
	

	
enemies = XGroup()
itemz = XGroup()
projectiles = XGroup()	



